#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Radha Panini's article “Using the Fractal Dimension Index – Trading Systems and Fractals”, 
	/// appearing the March 2007 edition of Technical Analysis of Stocks & Commodities magazine, 
	/// discusses the benefits of applying fractal analysis to a few well-known trading systems. 
	/// Panini focuses primarily on an indicator called the Fractal Dimension Index (FDI) which is 
	/// applied as a filter for the trade entries generated by the sample trading systems. The FDI 
	/// is used to measure the fractal dimension of a price series. Typically, its values range 
	/// between 1-2. In this article, 1.5 is the level used to filter trade entries. FDI values 
	/// above 1.5 indicate that the market is trading in a range while values below 1.5 indicate 
	/// that the market is trending.
    /// </summary>
    [Description("Fractal Dimension Index")]
    public class FDI : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int indicatorPeriod = 100; // Default setting for Period
		
		private double calculatedFDI;
		private double HurstExponent;
		private	double StdDeviationPeriod;
		private	double RangeOfCulmativeDeviation;
			
		
		private DataSeries LogarithmicReturn;
		private DataSeries AverageReturn;
		private DataSeries CulmativeDeviation;
		private DataSeries LogarithmicReturnMinusAverageReturn;
		
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Index"));
            
			Add(new Line(System.Drawing.Color.Red, 1.5, "RandomWalk"));
			
			CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			
			LogarithmicReturn = new DataSeries(this);
			AverageReturn = new DataSeries(this);
			CulmativeDeviation = new DataSeries(this);
			LogarithmicReturnMinusAverageReturn = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            
			
			
			if(CurrentBar < indicatorPeriod)
				return;
			else
				{
					double sum =0;
			
					LogarithmicReturn.Set(Math.Log(Input[0]/Input[1]));
					
					for (int barsBack = indicatorPeriod -1; barsBack >= 0; barsBack--)
						sum += LogarithmicReturn[barsBack];
					
					
				
					AverageReturn.Set(sum/indicatorPeriod);
					
					
					sum = 0;
				
					
		
					for(int barsBack = indicatorPeriod -1; barsBack >= 0; barsBack--)
						sum += (LogarithmicReturn[barsBack] - AverageReturn[barsBack]);
					
					CulmativeDeviation.Set(sum);
				
					
				RangeOfCulmativeDeviation = MAX(CulmativeDeviation, indicatorPeriod)[0] - MIN(CulmativeDeviation, indicatorPeriod)[0];
				StdDeviationPeriod = StdDev(LogarithmicReturn, indicatorPeriod)[0];

				HurstExponent = Math.Log(RangeOfCulmativeDeviation/StdDeviationPeriod)/Math.Log(indicatorPeriod/2);

				calculatedFDI = 2-HurstExponent;
				
				Index.Set(calculatedFDI);
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Index
        {
            get { return Values[0]; }
        }

        [Description("Period for getting Fractal Dimension")]
        [Category("Parameters")]
        public int IndicatorPeriod
        {
            get { return indicatorPeriod; }
			set { indicatorPeriod = Math.Max(1, value); }
        }
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FDI[] cacheFDI = null;

        private static FDI checkFDI = new FDI();

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public FDI FDI(int indicatorPeriod)
        {
            return FDI(Input, indicatorPeriod);
        }

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public FDI FDI(Data.IDataSeries input, int indicatorPeriod)
        {
            checkFDI.IndicatorPeriod = indicatorPeriod;
            indicatorPeriod = checkFDI.IndicatorPeriod;

            if (cacheFDI != null)
                for (int idx = 0; idx < cacheFDI.Length; idx++)
                    if (cacheFDI[idx].IndicatorPeriod == indicatorPeriod && cacheFDI[idx].EqualsInput(input))
                        return cacheFDI[idx];

            FDI indicator = new FDI();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.IndicatorPeriod = indicatorPeriod;
            indicator.SetUp();

            FDI[] tmp = new FDI[cacheFDI == null ? 1 : cacheFDI.Length + 1];
            if (cacheFDI != null)
                cacheFDI.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheFDI = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FDI FDI(int indicatorPeriod)
        {
            return _indicator.FDI(Input, indicatorPeriod);
        }

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public Indicator.FDI FDI(Data.IDataSeries input, int indicatorPeriod)
        {
            return _indicator.FDI(input, indicatorPeriod);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FDI FDI(int indicatorPeriod)
        {
            return _indicator.FDI(Input, indicatorPeriod);
        }

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public Indicator.FDI FDI(Data.IDataSeries input, int indicatorPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FDI(input, indicatorPeriod);
        }

    }
}
#endregion
